"""A folder that acts like the root of a constrained portal"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo

from Products.Archetypes.public import *

from Products.PloneMiniPortal.interfaces import IMiniPortal

from Products.ATContentTypes.content.folder import ATFolder
from Products.ATContentTypes.content.folder import finalizeATCTSchema
from Products.ATContentTypes.configuration import zconf

#from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder

MiniPortalSchema = ATFolder.schema.copy() + Schema((

        TextField('text',
                required=True,
                searchable=True,
                primary=True,
                storage = AnnotationStorage(migrate=True),
                validators = ('isTidyHtmlWithCleanup',),
                #validators = ('isTidyHtml',),
                default_content_type = zconf.ATDocument.default_content_type,
                default_output_type = 'text/x-html-safe',
                allowable_content_types = zconf.ATDocument.allowed_content_types,
                widget = RichWidget(
                            description = "",
                            description_msgid = "help_body_text",
                            label = "Body Text",
                            label_msgid = "label_body_text",
                            rows = 25,
                            i18n_domain = "plone",
                            allow_file_upload = zconf.ATDocument.allow_document_upload,
                ),
        ),

        BooleanField('miniPortalParent',
            default=True,
            widget=LabelWidget(
                visible={'edit':'hidden','view':'hidden'},
            ),
        ),
        StringField('mpPublicContact',
            schemata='admin',
            required=1,
            widget=StringWidget(
                label='Public E-Mail Contact',
                description='Submissions to the contact-info form will be sent here.',
            ),
        ),
        StringField('mpAdminContact',
            schemata='admin',
            required=1,
            widget=StringWidget(
                label='Admin E-Mail Contact',
                description="""
                    Contact information, including email, for a person responsible for this site.
                    This information is not part of the site web pages, but will be used by the
                    host administrator if there are questions about the site.
                    <strong>Please Note:</strong>
                    if this information is missing, or if mail to this address is not answered
                    promptly, your site may be deleted without notice.                """,
            ),
        ),
        StringField('mpLicense',
            schemata='admin',
            default='',
            vocabulary='_get_selection_vocab_license',
            enforceVocabulary=1,
            widget=SelectionWidget(
                label='License',
                description="""
                    Choose a <a href="http://creativecommons.org/" target="_blank">Creative Commons</a> license to allow
                    republication or your content.
                """,
            ),
        ),
        BooleanField('mpCreditSponsor',
            schemata='admin',
            default=True,
            widget=BooleanWidget(
                label='Credit DCN',
                description='If checked, will show a message at page bottom crediting DCN for site assistance.',
            ),
        ),
        BooleanField('mpShowRightCol',
            schemata='style',
            default=True,
            widget=BooleanWidget(
            label='Show Right Column',
            ),
         ),
        StringField('mpBackgroundColor',
            schemata='style',
            default='White',
            widget=StringWidget(
                label='Page Background Color',
                description='Description for <a href="http://www.dcn.org/">Page Background Color</a>',
            ),
        ),
        StringField('mpGlobalBackgroundColor',
            schemata='style',
            default='#dee7ec',
            widget=StringWidget(
                label='Contrast Background Color',
                description='Background color for boxed elements like column headers',
            ),
        ),
        StringField('mpGlobalBorderColor',
            schemata='style',
            default='#8cacbb',
            widget=StringWidget(
                label='Contrast Border Color',
                description='Border color for boxed elements like column headers',
            ),
        ),
        StringField('mpEvenRowBackgroundColor',
            schemata='style',
            default='#eef3f5',
            widget=StringWidget(
                label='Alternating Row Shading',
                description="""
                    This color will be used as the background color in alternate table rows.
                    A lighter version of the Contrast Background Color is a good choice.
                """,
            ),
        ),
#        StringField('mpFontFamily',
#            default='"Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif',
#            widget=StringWidget(
#                label='Body Font Family',
#            ),
#        ),
#        StringField('mpFontBaseSize',
#            default='69%',
#            widget=StringWidget(
#                label='Baseline Font Size',
#                description="Must be a percentage. All text sizes change proportionally to this size.",
#            ),
#        ),
#        StringField('mpHeadingFontFamily',
#            default='"Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif',
#            widget=StringWidget(
#                label='Headline Font Family',
#            ),
#        ),
        TextField('mpExtraCSS',
            schemata='style',
            default="""/* Set body and headline text color */\nh1, h2, h3, h4, h5, body {color: #303}""",
            widget=TextAreaWidget(
                label='Custom CSS',
                description="""
                  Cascading Style Sheet style-tag definitions.
                  Whatever you specify will override the Simple Sites default style sheet.
                  Please note: you do not need to include the &lt;style&gt;&lt;/style&gt; tags.
                  They are supplied for you.
                """,
            ),
        ),
        ImageField('mpBannerImage',
            schemata='banner',
            widget=ImageWidget(
                label='Banner Image',
                description="""
                    An optional graphic that will be displayed at the top of each page in place of a text title.
                    Maximum dimensions should be 550px wide, 100px high.
                """,
            ),
        ),
        TextField('mpFooter',
            schemata='footer',
            storage = AnnotationStorage(migrate=True),
            validators = ('isTidyHtmlWithCleanup',),
            #validators = ('isTidyHtml',),
            default_content_type = zconf.ATDocument.default_content_type,
            default_output_type = 'text/x-html-safe',
            allowable_content_types = zconf.ATDocument.allowed_content_types,
            widget = RichWidget(
                description = "This text will display at the bottom of all your pages. Most commonly used for name, address, contact info.",
                description_msgid = "help_footer_text",
                label = "Footer Text",
                label_msgid = "label_body_text",
                rows = 7,
                i18n_domain = "plone",
                allow_file_upload = False,
            ),
        ),
    ),)

finalizeATCTSchema(MiniPortalSchema, moveDiscussion=False)

MiniPortalSchema['description'].widget.description = 'A short summary of the site'


class MiniPortal(ATFolder):
    """
    A folder that acts like the root of a constrained portal
    """

    # Standard content type setup
    portal_type = meta_type = 'MiniPortal'
    archetype_name = 'Mini Portal'
    content_icon = 'MiniPortal.gif'
    schema = MiniPortalSchema
    typeDescription= 'A folder that acts like the root of a constrained portal'
    typeDescMsgId  = 'MiniPortal_description_edit'


    # Allow images and files to be uploaded into this containerish object
    allowed_content_types = ['Document', 'Folder', 'Image', 'File']

    # Set up our views - these are available from the 'display' menu
    default_view   = 'document_view'
    immediate_view = 'document_view'
    suppl_views = ('miniportal_view_preview', 'miniportal_view_float')

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    __implements__ = ATFolder.__implements__ + \
                     (IMiniPortal,)

    # Get the standard actions (tabs)
    # Subtract 'Sharing'
    actions = [act for act in ATFolder.actions if act['id'] not in ['local_roles']]

    # enable FTP/WebDAV and friends
    PUT = ATFolder.PUT

    def _get_selection_vocab_license(self):
        return DisplayList((
            ('','None'),
            ('mp_cc_a','Creative Commons Attribution'),
            ('mp_cc_a_sa','Creative Commons Attribution-ShareAlike'),
            ('mp_cc_a_nc','Creative Commons Attribution-NonCommercial'),
            ('mp_cc_a_nc_sa','Creative Commons Attribution-NonCommercial-ShareAlike'),
            ))

registerType(MiniPortal)
