"""A folder that acts like the root of a constrained portal"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

# Change these to credit your own organization
CREDIT_LABEL = 'Credit DCN',
CREDIT_DESCRIPTION = 'If checked, will show a message at page bottom crediting DCN for site assistance.',

from AccessControl import ClassSecurityInfo

from Products.Archetypes.public import *

from Products.PloneMiniPortal.interfaces import IMiniPortal

from Products.ATContentTypes.content.document import ATDocument
from Products.ATContentTypes.content.document import finalizeATCTSchema
from Products.ATContentTypes.configuration import zconf

from Products.CMFCore.permissions import ModifyPortalContent, ManagePortal, ReplyToItem
from Products.CMFCore.utils import getToolByName

from Products.ATColorPickerWidget.ColorPickerWidget import ColorPickerWidget

#from Products.CMFPlone.interfaces.OrderedContainer import IOrderedContainer

colorHelp = """
<a href="mpColorHelp" target="color_help" onClick="window.open('mpColorHelp','color_help','width=400,height=400,menubar=yes,toolbar=yes,scrollbars=yes,resizable=yes').focus(); return false;"><strong>Color Help</strong></a>
"""

MiniPortalSchema = ATDocument.schema.copy() + Schema((
        StringField('mpPublicContact',
            schemata='admin',
            required=1,
            validators = ('isEmail',),
            widget=StringWidget(
                label='Public E-Mail Contact',
                description='Submissions to the contact-info form will be sent here.',
            ),
        ),
        StringField('mpAdminContact',
            schemata='admin',
            required=1,
            widget=StringWidget(
                label='Admin E-Mail Contact',
                description="""
                    Contact information, including email, for a person responsible for this site.
                    This information is not part of the site web pages, but will be used by the
                    host administrator if there are questions about the site.
                    <strong>Please Note:</strong>
                    if this information is missing, or if mail to this address is not answered
                    promptly, your site may be deleted without notice.
		    """,
            ),
        ),
        StringField('mpLicense',
            schemata='admin',
            default='',
            vocabulary='_get_selection_vocab_license',
            enforceVocabulary=1,
            widget=SelectionWidget(
                label='License',
                description="""
                    Choose a <a href="http://creativecommons.org/" target="_blank">Creative Commons</a> license to allow
                    republication of your content. If you don't wish to use a Creative Commons License, you may wish
		    to add a copyright notice to your footer.
                """,
            ),
        ),
        BooleanField('mpCreditSponsor',
            schemata='admin',
            default=True,
            widget=BooleanWidget(
	        label = 'Credit DCN',
		    description = 'If checked, will show a message at page bottom crediting DCN for site assistance.',
            ),
        ),
        StringField('mpPrefURL',
            schemata='admin',
            default='',
            validators = ('isURL',),
            widget=StringWidget(
                label='Preferred URL',
                description="""
                  If you wish to have your site in the automatic MiniPortal list, and you are using a
		  domain/host name other than groups.dcn.org, enter your preferred URL here. The URL
		  must be a valid web address, including "http://".
                """,
            ),
        ),
        BooleanField('mpAllowAnonReply',
            schemata='admin',
            default=True,
            widget=BooleanWidget(
	        label = 'Allow Anonymous Discussion',
                description="""
                    Do you wish to allow anonymous visitors to discuss content? This will only apply
                    if discussion is enabled for the item.
                """,
            ),
        ),
        BooleanField('mpShowLeftCol',
            schemata='layout',
            default=True,
            widget=BooleanWidget(
            label='Show Left Column',
            description="""
                The site's left column is used for navigation. Turn it off
                and the center-column text will expand to fill the space.
            """,
            ),
         ),
        BooleanField('mpShowRightCol',
            schemata='layout',
            default=True,
            widget=BooleanWidget(
            label='Show Right Column',
            description="""
                The right column of the site's home page is used for news and events portlets. Turn it off
                and the center-column text will expand to fill the space.
            """,
            ),
         ),
        BooleanField('mpShowSearch',
            schemata='layout',
            default=True,
            widget=BooleanWidget(
            label='Show Search Form',
            description="""
                When turned on, a site search form will show up in the upper-left corner of
                every page. If you don't have many pages in your site, this really isn't useful.
            """,
            ),
         ),
        BooleanField('mpShowTabs',
            schemata='layout',
            default=True,
            widget=BooleanWidget(
            label='Show Navigation Tabs',
            description="""
                When turned on, a navigation tabs will be displayed in a horizontal bar in the
                top-of-page banner. If it isn't helpful, turn it off.
            """,
            ),
         ),
        BooleanField('mpAutoTabs',
            schemata='layout',
            default=True,
            widget=BooleanWidget(
            label='Show Folders as Tabs',
            description="""
                When turned on, this causes your top-level folders to appear as navigation
                tabs. If you have many top-level folders, you'll want to turn this off to
		avoid visual clutter.
		Navigation tabs must be on for this to have effect.
            """,
            ),
         ),
        BooleanField('mpShowCrumbs',
            schemata='layout',
            default=True,
            widget=BooleanWidget(
            label='Show Breadcrumbs',
            description="""
                When turned on, a "you are here" line will appear under the tab area.
		This is a navigational device that can help orient users who find themselves
		deep in a site folder hierarchy. You may wish to turn off breadcrumb display
		if your site is very shallow (in form, not substance).
            """,
            ),
         ),
        BooleanField('mpShowLogin',
            schemata='layout',
            default=True,
            widget=BooleanWidget(
            label='Show Log-in Link',
            description="""
                When turned on, a "log in" link will appear in the shaded horizontal bar
                under the banner when the current viewer is not logged in. This is convenient
		when you're getting started, but may later invite confusion from viewers
		who may suppose your site has a members-only component.
		Note that you may always reach a log-in form by just adding "/edit" to the
		URL of your web site.
            """,
            ),
         ),
        StringField('mpBackgroundColor',
            schemata='style',
            default='White',
            validators = ('isCSSColor',),
            widget=ColorPickerWidget(
                label='Page Background Color',
                description='Overall background color for the page. %s' % colorHelp,
                allow_brightness=True,
            ),
        ),
        StringField('mpGlobalBackgroundColor',
            schemata='style',
            default='#dee7ec',
            validators = ('isCSSColor',),
            widget=ColorPickerWidget(
                label='Contrast Background Color',
                description='Background color for boxed elements like column headers. %s' % colorHelp,
                allow_brightness=True,
            ),
        ),
        StringField('mpGlobalBorderColor',
            schemata='style',
            default='#8cacbb',
            validators = ('isCSSColor',),
            widget=ColorPickerWidget(
                label='Contrast Border Color',
                description='Border color for boxed elements like column headers. %s' % colorHelp,
                allow_brightness=True,
            ),
        ),
        StringField('mpEvenRowBackgroundColor',
            schemata='style',
            default='#eef3f5',
            validators = ('isCSSColor',),
            widget=ColorPickerWidget(
                label='Alternating Row Shading',
                description="""
                    This color will be used as the background color in alternate table rows.
                    A slightly different shade of the Page Background Color is a good choice. %s
                """ % colorHelp,
                allow_brightness=True,
            ),
        ),
#        StringField('mpFontFamily',
#            default='"Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif',
#            widget=StringWidget(
#                label='Body Font Family',
#            ),
#        ),
#        StringField('mpFontBaseSize',
#            default='69%',
#            widget=StringWidget(
#                label='Baseline Font Size',
#                description="Must be a percentage. All text sizes change proportionally to this size.",
#            ),
#        ),
#        StringField('mpHeadingFontFamily',
#            default='"Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif',
#            widget=StringWidget(
#                label='Headline Font Family',
#            ),
#        ),
        TextField('mpExtraCSS',
            schemata='style',
            default="""/* Set body and headline text color */\nh1, h2, h3, h4, h5, body {color: #303}""",
            widget=TextAreaWidget(
                label='Custom CSS',
                description="""
                  Cascading Style Sheet style-tag definitions.
                  Whatever you specify will override the Simple Sites default style sheet.
                  Please note: you do not need to include the &lt;style&gt;&lt;/style&gt; tags.
                  They are supplied for you.
                """,
            ),
        ),
        ImageField('mpBannerImage',
            schemata='banner',
	    allowable_content_types=('image/gif','image/jpeg','image/png'),
            widget=ImageWidget(
                label='Banner Image',
                description="""
                    An optional graphic that will be displayed at the top of each page in place of a text title.
                    Maximum dimensions should be 550px wide, 100px high. Make sure to use a web-standard file
		    format (JPG, JPEG, PNG or GIF) for the image upload.
                """,
            ),
        ),
        TextField('mpFooter',
            schemata='footer',
            storage = AnnotationStorage(migrate=True),
            validators = ('isTidyHtmlWithCleanup',),
            #validators = ('isTidyHtml',),
            default_content_type = zconf.ATDocument.default_content_type,
            default_output_type = 'text/x-html-safe',
            allowable_content_types = zconf.ATDocument.allowed_content_types,
            widget = RichWidget(
                description = "This text will display at the bottom of all your pages. Most commonly used for name, address, contact info.",
                description_msgid = "help_footer_text",
                label = "Footer Text",
                label_msgid = "label_body_text",
                rows = 7,
                i18n_domain = "plone",
                allow_file_upload = False,
            ),
        ),
        BooleanField('mpShowHomeNews',
            schemata='news/events',
            default=False,
            widget=BooleanWidget(
            label='Show News on Homepage',
            description="""
                When turned on, your site's most recent published news items will show up on the homepage, immediately
                following the body text.
            """,
            ),
         ),
        IntegerField('mpHomeNewsLimit',
            schemata='news/events',
            default=5,
            widget=IntegerWidget(
                label='Homepage News Item Limit',
                description="""
                    Sets the maximum number of news items to be listed on the home page.
                """,
            ),
         ),
        BooleanField('mpShowNewsTab',
            schemata='news/events',
            default=True,
            widget=BooleanWidget(
            label='Show News Tab',
            description="""
                When turned on, a navigation tab will be added for your news item listing. This
		only matters if the display of tabs is turned on (see the layout sub-form).
            """,
            ),
         ),
        BooleanField('mpShowNewsPortlet',
            schemata='news/events',
            default=True,
            widget=BooleanWidget(
            label='Show News Portlet',
            description="""
                When turned on, a brief listing of news items will show in a right-column portlet.
            """,
            ),
         ),
        IntegerField('mpNewsPortletLimit',
            schemata='news/events',
            default=5,
            widget=IntegerWidget(
                label='News Portlet Item Limit',
                description="""
                    Sets the maximum number of news items to be listed in the news portlet.
                """,
            ),
         ),
        BooleanField('mpShowEventsTab',
            schemata='news/events',
            default=True,
            widget=BooleanWidget(
            label='Show Events Tab',
            description="""
                When turned on, a navigation tab will be added for your event listing. This
		only matters if the display of tabs is turned on (see the layout sub-form).
            """,
            ),
         ),
        BooleanField('mpShowEventsPortlet',
            schemata='news/events',
            default=True,
            widget=BooleanWidget(
            label='Show Events Portlet',
            description="""
                When turned on, a brief listing of events will show in a right-column portlet.
            """,
            ),
         ),
        IntegerField('mpEventsPortletLimit',
            schemata='news/events',
            default=5,
            widget=IntegerWidget(
                label='Events Portlet Item Limit',
                description="""
                    Sets the maximum number of events to be listed in the events portlet.
                """,
            ),
         ),
    ),)

finalizeATCTSchema(MiniPortalSchema, folderish=True, moveDiscussion=False)

MiniPortalSchema['text'].widget.label = 'Home Page Body Text'
MiniPortalSchema['text'].default = """<p>Log in and edit your site to create homepage content.</p>"""
MiniPortalSchema['description'].widget.description = 'A short summary of the site'
MiniPortalSchema['allowDiscussion'].widget.visible['edit'] = 'invisible'
MiniPortalSchema['excludeFromNav'].schemata = 'admin'
MiniPortalSchema['excludeFromNav'].widget.label = "Exclude From Site List"
MiniPortalSchema['excludeFromNav'].widget.description = """
  Check this if you do not wish your site listed in DCN's automatically generated list
  of group sites.
  """
MiniPortalSchema.moveField('excludeFromNav', after='mpCreditSponsor')

class MiniPortal(OrderedBaseFolder, ATDocument):
    """A folder that contains a simplified portal"""

    # Standard content type setup
    portal_type = meta_type = 'MiniPortal'
    archetype_name = 'Mini Portal'
    content_icon = 'MiniPortal.gif'
    schema = MiniPortalSchema
    typeDescription= 'A folder that contains a simplified portal'
    typeDescMsgId  = 'MiniPortal_description_edit'

    # A quick semaphor that we can use to test via aquisition for embedded content
    miniPortalParent = True


#    # Allow images and files to be uploaded into this containerish object
#    allowed_content_types = ['Document', 'Folder', 'Image', 'File', 'Link', 'RichDocument', 'News Item', 'Event', ]

    # Set up our views - these are available from the 'display' menu
    default_view   = 'miniportal_view'
    immediate_view = 'miniportal_view'
#    suppl_views = ('miniportal_view_preview', 'miniportal_view_float')

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    __implements__ = OrderedBaseFolder.__implements__ + \
                     ATDocument.__implements__ + \
                     (IMiniPortal,)

    # Get the standard actions (tabs)
    # Subtract 'Sharing'
    # actions = [act for act in ATDocument.actions if act['id'] not in ['local_roles']]
    # actions = [act for act in ATDocument.actions]

    actions = list(ATDocument.actions)

    # restrict sharing tab
    for act in actions:
	if act['id'] == 'local_roles':
	    act['permissions'] = (ManagePortal,)
    
    # add a 'Documentation' tab
    actions.append(
         {
          'id'          : 'mpDoc',
          'name'        : 'Documentation',
          'action'      : 'string:${object_url}/mpDocumentation',
          'permissions' : (ModifyPortalContent,),
          'visible'     : 1,
         },
    )

#    def canSetDefaultPage(self):
#        return False

    # enable FTP/WebDAV and friends
    PUT = ATDocument.PUT

    def _get_selection_vocab_license(self):
        mpt = getToolByName(self, 'portal_miniportals')
        options = mpt.license_options
        alist = []
        for lic in options:
            try:
                id, label = lic.split('|')
                alist.append( (id, label) )
            except ValueError:
                pass
        return DisplayList( alist )

    def at_post_create_script(self):
        self.invokeFactory(id='news_and_events', type_name='Folder')
        folder1 = getattr(self, 'news_and_events')
        folder1.setExcludeFromNav(True)
        folder1.setTitle('News and Events')
        folder1.reindexObject()
        
        # By default, let's allow anonymous replies; this
        # only makes sense because MiniPortals have no membership components.
        # So, discussion is either public or pretty much useless.
        self.setMpAllowAnonReply('1')

    def getRemoteUrl(self):
	""" return the preferred URL for the miniportal """

	return self.mpPrefURL

    # the following methods are meant to be used in an enclosed object
    # that is invoking the method by aquisition.
        
    def getMiniPortalObject(self):
        """return the enclosing MiniPortal object"""
        
        return self
        
    def getMiniPortalUrl(self):
        """return the enclosing MiniPortal object URL"""
        
        return self.absolute_url()
        
    def getMiniPortalPath(self):
        """return the enclosing MiniPortal object URL"""
        
        return '/'.join(self.getPhysicalPath())
        
    def getMiniPortalTitle(self):
        """return the enclosing MiniPortal object Title"""
        
        return self.title_or_id()

    #
    # accessor and mutator for mpAllowAnonReply

    def getMpAllowAnonReply(self):
        """Is ReplyToItem set for Anonymous"""
        
        for role in self.rolesOfPermission(ReplyToItem):
            if role.get('name','') == 'Anonymous':
                return role.get('selected','') == 'SELECTED'
        return False

    def setMpAllowAnonReply(self, value, **kw):
        """Set ReplyToItem for Anonymous"""
        
        otherRoles = []
        for role in self.rolesOfPermission(ReplyToItem):
            if role.get('selected','')=='SELECTED':
                name = role['name']
                if name != 'Anonymous':
                    otherRoles.append(name)
        if int(value):
            otherRoles.append('Anonymous')
        self.manage_permission( ReplyToItem,
                                otherRoles,
                                acquire=self.acquiredRolesAreUsedBy('Reply to item'), 
                                REQUEST=None )
        

registerType(MiniPortal)
